/*
    qtZenOscillo : Software for using DSO-2020 usb scope under Linux
    Copyright (C) 2015 Olivier Poitevin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact : code@zener.fr

*/

#ifndef DATATHREAD_H
#define DATATHREAD_H

#include <QObject>
#include <QThread>
#include <QVector>
#include <QMutex>
#include "usbcom.h"

class DataThread : public QThread
{
    Q_OBJECT
public:
    explicit DataThread(QObject *parent = 0);
    ~DataThread();
    bool initialize();
    void doStop();

    bool setSpeed(int mantissa,int unit);
    void setCh1Activated(bool value);
    void setCh2Activated(bool value);
    bool setCh1Range(int mV);
    bool setCh2Range(int mV);
    void setFastMode(bool value);

signals:
    void dataReady(QVector<double> chan1,QVector<double> chan2,int nbToDisplay);

protected:
    void run();

public slots:

private:
    bool trigNow();
    bool stopThis;
    bool stopped;
    bool usbOpened;
    UsbCom usbCom;
    QMutex usbMtx;
    QMutex bulkDataSizeMtx;
    // configuration :
    int speedM; // mantissa
    int speedU; // unit (0 for ns, 1:us, 2:ms, 3:s)
    bool ch1Activated; // true if ch1 Activated
    bool ch2Activated; // true if ch2 Activated
    int ch1Range; // in mV
    int ch2Range; // in mV
    bool calibrated;
    double ch1Coef;
    double ch2Coef;
    int timeNbSmp; // number of samples to have correct time
    int bulkDataSize;
    bool fastMode;
};

#define UNIT_NS 0
#define UNIT_US 1
#define UNIT_MS 2
#define UNIT_S  3

#endif // DATATHREAD_H
